#/bin/sh

#Add the tools in this directory to the path
PATH=$PATH:/Insight/bin:/Insight/sbin
IDENTITY_LINK=/Insight/identity
IDENTITY_CANDIDATES=/Insight/sbin/identity_tools

if [ "$2" ];
then
	SERVER=$2
else
	echo "You must provide the servername of the end-of-line commissioning server."
	echo "Usage:"
	echo "eol.sh <server URL> <output_id_key_file>"
	exit 1
fi

if [ "$3" ];
then
    KEY_FILE=$3
else
    echo "You must provide a file to store the assigned identity key in."
    echo "Usage:"
    echo "eol.sh <server URL> <output_id_key_file>"
    exit 1
fi

# Find the proper insight identity program
for id_tool in $IDENTITY_CANDIDATES/*
do
	if [ -f $id_tool ];
	then
		echo Trying "$id_tool"
		serial=$($id_tool -s)
		if [ $? -eq 0 ];
		then
			SELECTED_ID_TOOL=$id_tool
		fi
	fi
done

if [ -z $SELECTED_ID_TOOL ];
then
	echo "No applicable ID tool could be found!"
	exit 1
fi

# Create the identity tool link
ln -s $SELECTED_ID_TOOL $IDENTITY_LINK

# Get the proper serial number
SERIAL_NUMBER=$($SELECTED_ID_TOOL -s)

# Create a JSON package to request an ID key from the end-of-line commissioning server.
reqTmp=$(mktemp)
echo "serial:"$SERIAL_NUMBER > $reqTmp
request=$(./2json < $reqTmp)
curl -H "Content-Type: application/json" -X POST -d "$request" $SERVER/identity > $reqTmp
if [ $? -ne 0 ];
then
	exit 1
fi

./jextract --data $reqTmp --member key > $KEY_FILE
if [ $? -ne 0 ];
then
	exit 1
fi

