#!/bin/sh

#Add the tools in this directory to the path
PATH=$PATH:/Insight/bin:/Insight/sbin

if [ "$2" ];
then
	SERVER=$2
else
	echo "You must provide a servername for the InsightCORE server."
	echo "Usage:"
	echo "loadUpgrade_network.sh <server URL> <ID key file> <machine type> <serial number> <package output directory>"
	exit 1
fi

if [ "$3" ];
then
	ID_KEY=$3
else
	echo "You must provide the location of the device identity key to authenticate the device."
	echo "Usage:"
	echo "loadUpgrade_network.sh <server URL> <ID key file> <machine type> <serial number> <package output directory>"
	exit 1
fi

if [ "$4" ];
then
	MACH_TYPE=$4
else
	echo "You must provide a machine type to download the upgrade for."
	echo "Usage:"
	echo "loadUpgrade_network.sh <server URL> <ID key file> <machine type> <serial number> <package output directory>"
	exit 1
fi

if [ "$5" ];
then
	SERIAL_NUMBER=$5
else
	echo "You must provide a the machine serial number."
	echo "Usage:"
		echo "loadUpgrade_network.sh <server URL> <ID key file> <machine type> <serial number> <package output directory>"
	exit 1
fi

if [ "$6" ];
then
	OUTPUT_DIR=$6
else
	echo "You must provide an output directory to stream upgrade contents to."
	echo "Usage:"
	echo "loadUpgrade_network.sh <server URL> <ID key file> <machine type> <serial number> <package output directory>"
	exit 1
fi


#Get the upgrade package information from the server.
upgradeJSONTmp=$(mktemp)
wget -O $upgradeJSONTmp $SERVER/version/$MACH_TYPE

if [ $? -ne 0 ];
then
	echo "Could not retrieve version information from the server!"
	exit 1
fi

#extract and compare the version information.
upgradeVerTmp=$(mktemp)
./jextract --data $upgradeJSONTmp --member ver > $upgradeVerTmp
localVersion=$(cat $upgradeVerTmp)
./version -t $localVersion

if [ $? -eq 1 ];
then
	echo "Local code version $localVersion is down-level, continuing upgrade..."
else
	echo "Local code version $localVersion is up-to-date with server code. Cancelling upgrade."
	exit 1
fi

#Create the required authentication packet.
authTmp=$(mktemp)
digestTmp=$(mktemp)

salt=$(openssl rand -base64 32)

key=$(cat $ID_KEY)
key=${key//$'\n'/} #Remove all newlines from the key

printf $MACH_TYPE$SERIAL_NUMBER$salt > $digestTmp

hmac=$(openssl dgst -sha1 -hex -hmac $key < $digestTmp | sed 's/^.* //')

echo "machtype:"$MACH_TYPE > $authTmp
echo "serialNumber:"$SERIAL_NUMBER >> $authTmp
echo "salt:"$salt >> $authTmp
echo "hmac:"$hmac >> $authTmp

packet=$(./2json < $authTmp)

#Request the passfile from the server with the credentials just created.
passfile=$(mktemp)
curl -H "Content-Type: application/json" -X POST -d "$packet" $SERVER/passphrase > $passfile

#Download, decrypt and untar into the provided output directory. Do the operation in-place to prevent RAM wastage in storing the encrypted/compressed file only to decompress it into additional memory
packageFileTmp=$(mktemp)
./jextract --data $upgradeJSONTmp --member package > $packageFileTmp
packageFile=$(cat $packageFileTmp)

current_dir=$(pwd)
cd $OUTPUT_DIR
wget $SERVER/packages/$packageFile -O - -q 2>/dev/null | openssl bf -d -pass file:$passfile | tar -xzvf -
if [ $? -ne 0 ];
then
	echo "An error occurred streaming the package file into the output directory $OUTPUT_DIR"
	exit 1
fi

exit

