#!/bin/sh

#Add the tools in this directory to the path
PATH=$PATH:/Insight/bin:/Insight/sbin

IDENTITY_TOOL=/Insight/identity
CONFIG=/Insight/config.json
if [ ! -e $CONFIG ];
then
	echo "No configuration file ($CONFIG) found!"
	exit 1
fi

if [ "$2" ];
then
	UPGRADE_TYPE=$2
else
	echo "You must indicate the type of upgrade!"
	echo "Usage:"
	echo "loadUpgrade_network.sh <type> <USB Directory>"
	echo "Valid options for <type> are:"
	echo "'usb': The upgrade will be loaded by USB."
	echo "'network': The upgrade will be loaded through the network."
	echo "<USB Directory>: The directory to output the USB device."
	exit 1
fi

if [ "$3" ];
then
	USB_DIR=$3
fi

#Retrieve the identity key. If it doesn't exist, run the end-of-line script to acquire it.
ID_KEY_FILE=$(jextract --data $CONFIG --member id-key)
if [ ! -f $ID_KEY_FILE ];
then
	EOL_SERVER=$(jextract --data $CONFIG --member eol-server)
	eol.sh -- $EOL_SERVER $ID_KEY_FILE
	
	if [ $? -ne 0 ];
	then
		exit 1
	fi
fi

#Get machine type and serial number from the identity tool
MACHINE_TYPE=$($IDENTITY_TOOL -m)
MACHINE_SERIAL=$($IDENTITY_TOOL -s)

#make a (RAM drive) directory to deploy the upgrade package into - no need to excessively write flash.
upgradeRoot=/upgrade
LOCK_FILENAME=upgrade.lock
LOCK_FILE=$upgradeRoot/$LOCK_FILENAME
LOCK_FD=200
ramDriveSize=64M

mkdir -p $upgradeRoot
mount -t tmpfs -o size=$ramDriveSize tmpfs $upgradeRoot || exit 1

#Lock the lock file to prevent other upgrade scripts from running at the same time, exit if there are others.
(
    flock -nx $LOCK_FD || exit 1

	#Download the package.
	if [ "$UPGRADE_TYPE" == "network" ];
	then
		echo "Selected network upgrade source."
		
		#Use the network script to download the network upgrade
		SERVER=$(jextract --data $CONFIG --member upgrade-server)
		./loadUpgrade_network.sh -- $SERVER $ID_KEY_FILE $MACHINE_TYPE $MACHINE_SERIAL $upgradeRoot
		if [ $? -ne 0 ];
		then
			echo "Upgrade failed..."
			exit 1
		fi
	elif [ "$UPGRADE_TYPE" == "usb" ];
	then
		echo "Selected USB upgrade source."
		
		if [ ! $USB_DIR ];
		then
			echo "You must indicate the path to the USB drive to upgrade from."
		fi
		
		./loadUpgrade_USB.sh -- upgrade $ID_KEY_FILE $MACHINE_TYPE $MACHINE_SERIAL $upgradeRoot $USB_DIR
		if [ $? -ne 0 ];
		then
			echo "Upgrade failed..."
			exit 1
		fi
	else
		echo "You must specify 'network' or 'usb' as the upgrade type!"
		exit 1
		flock -u $LOCK_FD
		exit $?
	fi
	
	#Run the installation script	
	cd $upgradeRoot
	if [ -e "install.sh" ]
	then
		   ./install.sh INSTALL
		   if [ $? -ne 0 ];
		   then
		           echo "Upgrade Failed!"
		           $INSIGHT_INIT_SCRIPT start
		           rm -rf $NEXT_VERSION_FILE
		           exit $?
		   fi
	fi


	#finished with extraction and installation - close the RAM drive.
	cd /
	umount $upgradeRoot
	rmdir $upgradeRoot

	#Restart the machine to ensure that all updates take place
	flock -u $LOCK_FD
#	shutdown -r now

) 200>$LOCK_FILE

